/***
Performs an initial call to eWriteNames to write configuration values, and then
calls eWriteNames and eReadNames repeatedly in a loop.

For documentation on register names to use, see the T-series Datasheet or the
Modbus Map:

https://labjack.com/support/datasheets/t-series
https://labjack.com/support/software/examples/modbus/modbus-map

***/
import java.io.DataInputStream;
import java.text.DecimalFormat;
import com.sun.jna.ptr.IntByReference;
import com.labjack.LJM;
import com.labjack.LJMException;

public class WriteReadLoopWithConfig {

	public static void main(final String[] args) {
		try {
			IntByReference handleRef = new IntByReference(0);
			int handle = 0;
			int deviceType = 0;
			int numFrames = 0;
			String[] aNames;
			double[] aValues;
			IntByReference errAddr = new IntByReference(-1);
			int intervalHandle = 1;
			IntByReference skippedIntervals = new IntByReference(0);

			//If arguments are passed to the application, the first argument
			//specifies how many times to loop. If an argument is not passed,
			//will loop until the Enter key is pressed.
			int loopAmount = 0;
			boolean infLoop = true;
			String loopStr = "\nStarting read loop. Press Enter to stop.";
			if(args.length > 0) {
				try {
					loopAmount = new Double(args[0]).intValue();
					infLoop = false;
					loopStr = "\nStarting " + loopAmount + " read loops.";
				}
				catch(Exception ex) {
					throw new Exception("Invalid first argument \"" + args[0]
							+ "\". This specifies how many times to loop and "
							+ "needs to be a number.");
				}
			}

			//Open first found LabJack

			//Any device, Any connection, Any identifier
			LJM.openS("ANY", "ANY", "ANY", handleRef);

			//T7 device, Any connection, Any identifier
			//LJM.openS("T7", "ANY", "ANY", handleRef);

			//T4 device, Any connection, Any identifier
			//LJM.openS("T4", "ANY", "ANY", handleRef);

			//Any device, Any connection, Any identifier
			//LJM.open(LJM.Constants.dtANY, LJM.Constants.ctANY, "ANY",
			//		handleRef);

			handle = handleRef.getValue();

			LJMUtilities.printDeviceInfo(handle);

			deviceType = LJMUtilities.getDeviceType(handle);

			//Setup and call eWriteNames for AIN0 (all devices) and
			//digital I/O (T4 only) configuration.
			if(deviceType == LJM.Constants.dtT4) {
				//LabJack T4 configuration

				//Set FIO5 (DIO5) and FIO6 (DIO6) lines to digital I/O.
				//    DIO_INHIBIT = 0xF9F, b111110011111.
				//                  Update only DIO5 and DIO6.
				//    DIO_ANALOG_ENABLE = 0x000, b000000000000.
				//                        Set DIO5 and DIO6 to digital I/O (b0).
				aNames = new String[] { "DIO_INHIBIT", "DIO_ANALOG_ENABLE" };
				aValues = new double[] { 0xF9F, 0x000 };
				numFrames = aNames.length;
				LJM.eWriteNames(handle, numFrames, aNames, aValues,
						errAddr);

				//AIN0:
				//    The T4 only has single-ended analog inputs.
				//    The range of AIN0-AIN3 is +/-10 V.
				//    The range of AIN4-AIN11 is 0-2.5 V.
				//    Resolution index = 0 (default)
				//    Settling = 0 (auto)
				aNames = new String[] { "AIN0_RESOLUTION_INDEX",
						"AIN0_SETTLING_US" };
				aValues = new double[] { 0, 0 };
			}
			else {
				//LabJack T7 and other devices configuration.

				//AIN0:
				//    Negative Channel = 199 (Single-ended)
				//    Range = +/-10 V
				//    Resolution index = 0 (default)
				//    Settling = 0 (auto)
				aNames = new String[] { "AIN0_NEGATIVE_CH", "AIN0_RANGE",
						"AIN0_RESOLUTION_INDEX", "AIN0_SETTLING_US" };
				aValues = new double[] { 199, 10, 0, 0 };
			}
			numFrames = aNames.length;
			LJM.eWriteNames(handle, numFrames, aNames, aValues, errAddr);

			System.out.println("\nSet configuration:");
			for(int i = 0; i < numFrames; i++) {
				System.out.println("  " + aNames[i] + " : " + aValues[i]);
			}

			System.out.println(loopStr);
			DataInputStream ins = new DataInputStream(System.in);
			int it = 0;
			double dacVolt = 0.0;
			int fioState = 0;
			LJM.startInterval(intervalHandle, 1000000);
			while(ins.available() <= 0) {
				//Setup and call eWriteNames to write to DAC0, and FIO5 (T4)
				//or FIO1 (T7 and other devices).
				//DAC0 will cycle ~0.0 to ~5.0 volts in 1.0 volt increments.
				//FIO5/FIO1 will toggle output high (1) and low (0) states.
				if (deviceType == LJM.Constants.dtT4) {
					aNames = new String[] { "DAC0", "FIO5" };
				}
				else {
					aNames = new String[] { "DAC0", "FIO1" };
				}
				dacVolt = it % 6.0;  //0-5
				fioState = it % 2;  //0 or 1
				aValues = new double[] { dacVolt, (double)fioState };
				numFrames = aNames.length;
				LJM.eWriteNames(handle, numFrames, aNames, aValues, errAddr);

				System.out.print("\n\neWriteNames :");
				for(int i = 0; i < numFrames; i++) {
					System.out.print(" " + aNames[i] + " = "
							+ String.format("%.4f", aValues[i]) + ", ");
				}

				//Setup and call eReadNames to read AIN0, and FIO6 (T4)
				//or FIO2 (T7 and other devices).
				if(deviceType == LJM.Constants.dtT4) {
					aNames = new String[] { "AIN0", "FIO6" };
				}
				else {
					aNames = new String[] { "AIN0", "FIO2" };
				}
				aValues = new double[] { 0, 0 };
				numFrames = aNames.length;
				LJM.eReadNames(handle, numFrames, aNames, aValues, errAddr);
				System.out.print("\neReadNames  :");
				for(int i = 0; i < numFrames; i++) {
					System.out.print(" " + aNames[i] + " = "
							+ String.format("%.4f", aValues[i]) + ", ");
				}

				//Stop after the user-specified number of loops.
				it++;
				if(!infLoop && it >= loopAmount) {
					break;
				}

				//Repeat every 1 second
				LJM.waitForNextInterval(intervalHandle, skippedIntervals);
				if(skippedIntervals.getValue() > 0) {
					System.out.print("\nSkippedIntervals: "
							+ String.format("%d", skippedIntervals.getValue()));
				}
			}

			//Close interval and device handles
			LJM.cleanInterval(intervalHandle);
			LJM.close(handle);
		}
		catch(LJMException le) {
			le.printStackTrace();
			LJM.closeAll();
		}
		catch(Exception e) {
			e.printStackTrace();
			LJM.closeAll();
		}
	}
}
